#!/bin/zsh
#
# .zshrc

zshrc_load_status () {
  echo -n "\r.zshrc load: $* ... \e[0K"
}

zshrc_load_status 'setting options'


ZSH_VERSION_TYPE=old
if [[ $ZSH_VERSION == 3.1.<6->* ||
      $ZSH_VERSION == 3.2.<->*  ||
      $ZSH_VERSION == 4.<->* ]]
then
    # pour le nouveau type de compltion ...
    if which zstyle > /dev/null ; then
        ZSH_VERSION_TYPE=new
    fi
fi


setopt \
     NO_all_export \
        always_last_prompt \
     NO_always_to_end \
        append_history \
        auto_cd \
        auto_list \
        auto_menu \
     NO_auto_name_dirs \
        auto_param_keys \
        auto_param_slash \
        auto_pushd \
        auto_remove_slash \
     NO_auto_resume \
     NO_bad_pattern \
        bang_hist \
     NO_beep \
        bg_nice \
	brace_ccl \
        correct_all \
     NO_bsd_echo \
        cdable_vars \
     NO_chase_links \
     NO_clobber \
        complete_aliases \
        complete_in_word \
        correct \
        correct_all \
        csh_junkie_history \
     NO_csh_junkie_loops \
     NO_csh_junkie_quotes \
     NO_csh_null_glob \
        equals \
        extended_glob \
        extended_history \
        function_argzero \
        glob \
     NO_glob_assign \
        glob_complete \
     NO_glob_dots \
        glob_subst \
        hash_cmds \
        hash_dirs \
        hash_list_all \
        hist_allow_clobber \
        hist_beep \
        hist_ignore_dups \
        hist_ignore_space \
     NO_hist_no_store \
        hist_verify \
     NO_hup \
     NO_ignore_braces \
     NO_ignore_eof \
        interactive_comments \
     NO_list_ambiguous \
     NO_list_beep \
        list_types \
        long_list_jobs \
        magic_equal_subst \
     NO_mail_warning \
     NO_mark_dirs \
     NO_menu_complete \
        multios \
        nomatch \
        notify \
     NO_null_glob \
        numeric_glob_sort \
     NO_overstrike \
        path_dirs \
        posix_builtins \
     NO_print_exit_value \
     NO_prompt_cr \
        prompt_subst \
        pushd_ignore_dups \
     NO_pushd_minus \
     NO_pushd_silent \
        pushd_to_home \
        rc_expand_param \
     NO_rc_quotes \
     NO_rm_star_silent \
     NO_sh_file_expansion \
        sh_option_letters \
        short_loops \
        sh_word_split \
     NO_single_line_zle \
     NO_sun_keyboard_hack \
        unset \
     NO_verbose \
        zle

if [[ $ZSH_VERSION_TYPE == 'new' ]]; then
  setopt \
        hist_expire_dups_first \
        hist_ignore_all_dups \
     NO_hist_no_functions \
     NO_hist_save_no_dups \
        inc_append_history \
        list_packed \
     NO_rm_star_wait
fi

if [[ $ZSH_VERSION == 3.0.<6->* || $ZSH_VERSION_TYPE == 'new' ]]; then
  setopt \
        hist_reduce_blanks
fi


zshrc_load_status 'setting environment'

# Some programs might find this handy.  Shouldn't do any harm.
export COLUMNS

[[ "$ZSH_VERSION_TYPE" == 'old' ]] || typeset -T INFOPATH infopath
typeset -U infopath
export INFOPATH
infopath=( 
          ~/local/info(N)
          /usr/info(N)
          $infopath
         )

zdotdir=${ZDOTDIR:-$HOME}

fpath=(
       {$zdotdir,$HOME}/{.zsh,lib/zsh}/{functions,scripts}(N) 

       $fpath

       # very old versions
       /usr/doc/zsh*/Functions(N)
      )

export PATH=$PATH:$HOME/bin:/usr/local/bin:/u0/local/bin:/usr/sbin:$HOME/cxoffice/bin:$HOME/crossover/bin

# Autoload all shell functions from all directories in $fpath that
# have the executable bit on (the executable bit is not necessary, but
# gives you an easy way to stop the autoloading of a particular shell
# function).

for dirname in $fpath; do
  fns=( $dirname/*~*~(N.x:t) )
  (( $#fns )) && autoload $fns
done
#[[ "$ZSH_VERSION_TYPE" == 'new' ]] || typeset -gU fpath

# Choose word delimiter characters in line editor

WORDCHARS=' '

#  Save a large history

HISTFILE=~/.zshhistory
HISTSIZE=3000
SAVEHIST=3000

# Maximum size of completion listing
# Only ask if line would scroll off screen
LISTMAX=0

# {{{ Watching for other users

LOGCHECK=60
WATCHFMT="[%B%t%b] %B%n%b has %a %B%l%b from %B%M%b"

# Pilot setting:
export PILOTRATE=115200
export PILOTPORT=/dev/pilot

# Auto Logout
#TMOUT=1800
#TRAPALRM () {
#  clear
#  echo Inactivity timeout on $TTY
#  echo
#  vlock -c
#  echo
#  echo Terminal unlocked. [ Press Enter ]
#}

# Prompts
# I'm not a big fan of the themable prompt system, so I'm not using it.
C_PROMPT=$'\e[0m'
C_DIR=$'\e[01;34m'
C_DEFAULT=$'\e[0m'
C_ALERT=$'\e[01;31m'
export PS1='%{'$C_PROMPT'%}%B[%! %n@%m]%(#.#.$)%b%{'$C_DEFAULT'%} ' 
export RPS1='%(?..%{'$C_ALERT'%}< %? > )%{'$C_DIR'%}%~%{'$C_DEFAULT'%}'
#RPROMPT='%~'
#PROMPT='%n@%m$ '



# Completions
if /bin/true && [[ "$ZSH_VERSION_TYPE" == 'new' ]]; then
  autoload -U compinit
  compinit -u # don't perform security check
else
  print "\nAdvanced completion system not found; ignoring zstyle settings."
  function zstyle { }
  function compdef { }

  # an antiquated, barebones completion system is better than nowt
  zmodload zsh/compctl
fi
##
## Enable the way cool bells and whistles.
##

# General completion technique
#zstyle ':completion:*' completer _complete _correct _approximate _prefix
zstyle ':completion:*' completer _complete _prefix
zstyle ':completion::prefix-1:*' completer _complete
zstyle ':completion:incremental:*' completer _complete _correct
zstyle ':completion:predict:*' completer _complete

# Completion caching
zstyle ':completion::complete:*' use-cache 1
zstyle ':completion::complete:*' cache-path ~/.zsh/cache/$HOST

# Expand partial paths
zstyle ':completion:*' expand 'yes'
zstyle ':completion:*' squeeze-slashes 'yes'

# Include non-hidden directories in globbed file completions
# for certain commands
#zstyle ':completion::complete:*' \
#  tag-order 'globbed-files directories' all-files 
#zstyle ':completion::complete:*:tar:directories' file-patterns '*~.*(-/)'

# Separate matches into groups
zstyle ':completion:*:matches' group 'yes'

# Describe each match group.
zstyle ':completion:*:descriptions' format "%B---- %d%b"

# Messages/warnings format
zstyle ':completion:*:messages' format '%B%U---- %d%u%b' 
zstyle ':completion:*:warnings' format '%B%U---- no match for: %d%u%b'
 
# Describe options in full
zstyle ':completion:*:options' description 'yes'
zstyle ':completion:*:options' auto-description '%d'

# }}}
# {{{ Simulate my old dabbrev-expand 3.0.5 patch 

zstyle ':completion:*:history-words' stop verbose
zstyle ':completion:*:history-words' remove-all-dups yes

myznol () {reply=(${$(znol -l | awk '{print $1}')%:})}


# Common usernames
users=(asegall bts cat cordelia cuss dalesag dan dmaze dnekorch doc growf jb jkujawa joelll justin kkc mg nygren pocky sarac sdrosch tla welborn)

zstyle ':completion:*' users $users

if [[ "$ZSH_VERSION_TYPE" == 'new' ]]; then
  : ${(A)_etc_hosts:=${(s: :)${(ps:\t:)${${(f)~~"$(</etc/hosts)"}%%\#*}##[:blank:]#[^[:blank:]]#}}}
else
  # Older versions don't like the above cruft
  _etc_hosts=()
fi

hosts=(
    "$_etc_hosts[@]"

    localhost

    # ftp sites
    sunsite.org.uk
)

# {{{ (user,host) pairs

# All my accounts:
#my_accounts=(
#    joe:
#    {joe,root}:mymachine.com
#    jbloggs:myothermachine.com
#)

# Other people's accounts:
#other_accounts=(
#    bob:
#    {fred,root}:hismachine.com
#    vera:hermachine.com
#)

zstyle ':completion:*:my-accounts' users-hosts $my_accounts
zstyle ':completion:*:other-accounts' users-hosts $other_accounts


# }}}
# {{{ (host, port, user) triples for telnet

#  telnet_hosts_ports_users=(
#    host1::user1
#    host2::user2
#    mail-server:{smtp,pop3}:
#    news-server:nntp:
#    proxy-server:8000:
#  )
zstyle ':completion:*:*:telnet:*' hosts-ports-users $telnet_hosts_ports_use

# }}}

# }}}
# {{{ Aliases and functions

zshrc_load_status 'aliases and functions'

# cope with galeon's Xt problem:
alias galeon 'LD_PRELOAD=/usr/X11R6/lib/libXt.so /usr/bin/galeon' 

# Turn off annoying gtimer splash screen
alias gtimer='gtimer -nosplash'

# Cope with StrictHostKeyChecking
alias loosessh='ssh -o "StrictHostKeyChecking no"'

# Template Stuff for #Darkness, #Hive, Foundation, DOA
#alias dvips='dvips -R '
alias dogame='export TEXINPUTS=.:$GAME/LaTeX//: ; export DVIPSHEADERS=$GAME/LaTeX/Postscript: ; cd $GAME'
#alias darkness='export GAME=/spare/Darkness; dogame'
#alias gmdark='~/bin/gm --game=/spare/Darkness'
#alias hive='export GAME=/spare/WretchedHive; dogame'
#alias gmhive='~/bin/gm --game=/spare/WretchedHive'
alias doa='export GAME=~/Games/DOA; dogame'
alias gmdoa='~/bin/gm --game=~/Games/DOA'
alias found='export GAME=~/Games/Foundation; dogame'
alias gmfound='~/bin/gm --game=~/Games/Foundation'


# dvorak/qwerty switching
alias asdf='xmodmap ~/.xmap.dvorak' 
alias aoeu='xmodmap ~/.xmap.qwerty' 

# {{{ which

# reverse unwanted aliasing of `which' by distribution startup
# files (e.g. /etc/profile.d/which*.sh); zsh's which is perfectly
# good as is.

alias which >&/dev/null && unalias which

# }}}
# {{{ run-help

alias run-help >&/dev/null && unalias run-help
autoload run-help

# }}}
# {{{ Restarting zsh or bash; reloading .zshrc or functions

bash () {
  NO_ZSH="yes" command bash "$@"
}

restart () {
  exec $SHELL "$@"
}

reload () {
  if [[ "$#*" -eq 0 ]]; then
    . $zdotdir/.zshrc
  else
    local fn
    for fn in "$@"; do
      unfunction $fn
      autoload -U $fn
    done
  fi
}
compdef _functions reload

# }}}
# {{{ ls aliases

if ls -F --color >&/dev/null; then
  alias ls='command ls -F --color'
elif ls -F >&/dev/null; then
  alias ls='command ls -F'
elif ls --color >&/dev/null; then
  alias ls='command ls --color'
fi

# jeez I'm lazy ...
alias l='ls -l'
alias ll='ls -l'
alias la='ls -la'
alias lsa='ls -a'
alias lsh='ls -d .*'
alias ld='ls -ld'
# damn, missed out lsd :-)
alias sl=ls # often screw this up

# }}}
# {{{ File management

# {{{ Changing/making/removing directory

# blegh
alias cd..='cd ..'
alias cd...='cd ../..'
alias cd....='cd ../../..'
alias cd.....='cd ../../../..'
alias cd/='cd /'

cd () {
  if   [[ "x$*" == "x..." ]]; then
    cd ../..
  elif [[ "x$*" == "x...." ]]; then
    cd ../../..
  elif [[ "x$*" == "x....." ]]; then
    cd ../../..
  elif [[ "x$*" == "x......" ]]; then
    cd ../../../..
  else
    builtin cd "$@"
  fi
}

alias md='mkdir -p'
alias rd=rmdir

alias dirs='dirs -v'
alias d=dirs

# Don't need this because auto_pushd is set
#alias pu=pushd

alias po=popd

# }}}
# {{{ Renaming

autoload zmv

# }}}

# }}}
# {{{ Job/process control

alias j='jobs -l'

# }}}
# {{{ History

alias h=history

# }}}
# {{{ Environment

alias ts=typeset
compdef _vars_eq ts

# }}}
# {{{ Terminal

# {{{ cls := clear

alias cls='clear'

# }}}

# }}}
# {{{ Other users

compdef _users lh

alias f=finger
compdef _finger f

# su changes window title, even if we're not a login shell
su () {
  command su "$@"
  cx
}

# }}}
# {{{ No spelling correction

alias man='nocorrect man'
alias mysql='nocorrect mysql'
alias mysqlshow='nocorrect mysqlshow'
alias p4='nocorrect p4'

# }}}
# {{{ X windows related

# {{{ Changing terminal window/icon titles

if [[ "$TERM" == xterm* ]]; then
  # Could also look at /proc/$PPID/cmdline ...
  which cx >&/dev/null || cx () { }
    # this was originally an ||, not an &&.  I'm not sure where the error's from.
  cx
fi

# }}}
# {{{ export DISPLAY=:0.0

alias sd='export DISPLAY=:0.0'

# }}}

# }}}
# {{{ Different CVS setups

# Sensible defaults
unset CVS_SERVER
export CVSROOT
export CVS_RSH=ssh

# for use with the scvs function
typeset -A _CVSROOTS _CVS_RSHS
_CVSROOTS=(
           # Put your own CVSROOTs here
           local             /usr/local/cvsroot
           mutt              :pserver:anonymous@ftp.guug.de:/home/roessler/cvs
)

_CVS_RSHS=(
           # Put your own CVS_RSHs here
          )

# }}}
# {{{ Other programs

# {{{ less

alias v=less

# }}}
# {{{ mutt

m () {
  cx mutt
  mutt "$@"
  cxx
}

# }}}
# {{{ editors

# emacs, windowed
e () {
  emacs "$@" &!
}

# emacs, fast, non-windowed (see also $VISUAL)
fe () {
  QUICK_EMACS=1 emacs -nw "$@"
}

# emacs, slow, non-windowed
se () {
  emacs -nw "$@"
}

alias pico='/usr/bin/pico -z'

# }}}
# {{{ remote logins

ssh () {
  command ssh "$@"
  cx
}

dsa >&/dev/null

alias sa=ssh-add

# }}}
# {{{ ftp

if which lftp >&/dev/null; then
  alias ftp=lftp
elif which ncftp >&/dev/null; then
  alias ftp=ncftp
fi

# }}}
# {{{ watching log files

alias tf='less +F'

# }}}

# }}}

# {{{ Global aliases

# WARNING: global aliases are evil.  Use with caution.

# {{{ Paging with less / head / tail

alias -g L='| less'
alias -g LS='| less -S'
alias -g EL='|& less'
alias -g ELS='|& less -S'
alias -g H='| head -20'
alias -g T='| tail -20'

# }}}
# {{{ Sorting / counting

alias -g C='| wc -l'

alias -g S='| sort'
alias -g US='| sort -u'
alias -g NS='| sort -n'
alias -g RNS='| sort -nr'

# }}}
# {{{ common filenames

alias -g DN=/dev/null

# }}}

# }}}

# }}}
# {{{ Key bindings 

zshrc_load_status 'key bindings'

bindkey -s '^X^Z' '%-^M'
bindkey '^[e' expand-cmd-path
bindkey -s '^X?' '\eb=\ef\C-x*'
bindkey '^[^I' reverse-menu-complete
bindkey '^X^N' accept-and-infer-next-history
bindkey '^[p' history-beginning-search-backward
bindkey '^[n' history-beginning-search-forward
bindkey '^[P' history-beginning-search-backward
bindkey '^[N' history-beginning-search-forward
bindkey '^W' kill-region
bindkey '^I' complete-word
bindkey '^[b' emacs-backward-word
bindkey '^[f' emacs-forward-word

# Fix weird sequence that rxvt produces
bindkey -s '^[[Z' '\t'

alias no=ls  # for Dvorak

# }}}
# {{{ Miscellaneous

zshrc_load_status 'miscellaneous'

# {{{ Hash named directories

#hash -d I3=/usr/src/redhat/RPMS/i386
#hash -d I6=/usr/src/redhat/RPMS/i686
#hash -d NA=/usr/src/redhat/RPMS/noarch
#hash -d SR=/usr/src/redhat/SRPMS
#hash -d SP=/usr/src/redhat/SPECS
#hash -d SO=/usr/src/redhat/SOURCES
#hash -d BU=/usr/src/redhat/BUILD
#hash -d CV=/usr/local/cvsroot
#hash -df

# }}}
# {{{ ls colours

if which dircolors >&/dev/null && [[ -e "${zdotdir}/.dircolors" ]]; then
  # show directories in yellow
  eval `dircolors -b $zdotdir/.dircolors`
fi

if [[ $ZSH_VERSION > 3.1.5 ]]; then
  zmodload -i zsh/complist

  zstyle ':completion:*' list-colors ''

  zstyle ':completion:*:*:kill:*:processes' list-colors \
    '=(#b) #([0-9]#)*=0=01;31'

  # completion colours
  zstyle ':completion:*' list-colors "$LS_COLORS"
fi  

# }}}

# Keychain
#if [[ -x "/usr/bin/keychain" ]]; then
#	export HOSTNAME=`uname -n`
#	if [[ -f "${HOME}/.p4ssh/identity" ]]; then
#	    /usr/bin/keychain --quiet ~/.ssh/id_rsa ~/.ssh/identity ~/.p4ssh/identity
#	else
#	    /usr/bin/keychain --quiet ~/.ssh/id_rsa ~/.ssh/identity
#	fi
#	. ~/.keychain/${HOSTNAME}-sh
#fi

# less configuration
eval $(lessfile)

# }}}

# {{{ Specific to xterms

if [[ "${TERM}" == xterm* ]]; then
  unset TMOUT
fi

# load host-specific files
if [[ -r ~/.zshrc.local ]]; then
  zshrc_load_status '.zshrc.local'
  . ~/.zshrc.local
fi

if [[ -r ~/.zshrc.${HOST%%.*} ]]; then
  zshrc_load_status ".zshrc.${HOST%%.*}"
  . ~/.zshrc.${HOST%%.*}
fi

export HOSTNAME=`uname -n`


# Clear up after status display
echo -n "\r"

